package java.util.logging;

import java.util.Iterator;
import java.security.AccessController;
import java.util.MissingResourceException;
import java.security.PrivilegedAction;
import java.util.function.Supplier;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.concurrent.CopyOnWriteArrayList;

public class Logger
{
    private static final Handler[] emptyHandlers;
    private static final int offValue;
    static final String SYSTEM_LOGGER_RB_NAME = "sun.util.logging.resources.logging";
    private static final Logger.LoggerBundle SYSTEM_BUNDLE;
    private static final Logger.LoggerBundle NO_RESOURCE_BUNDLE;
    private volatile LogManager manager;
    private String name;
    private final CopyOnWriteArrayList<Handler> handlers;
    private volatile Logger.LoggerBundle loggerBundle;
    private volatile boolean useParentHandlers;
    private volatile Filter filter;
    private boolean anonymous;
    private ResourceBundle catalog;
    private String catalogName;
    private Locale catalogLocale;
    private static final Object treeLock;
    private volatile Logger parent;
    private ArrayList<LogManager.LoggerWeakRef> kids;
    private volatile Level levelObject;
    private volatile int levelValue;
    private WeakReference<ClassLoader> callersClassLoaderRef;
    private final boolean isSystemLogger;
    public static final String GLOBAL_LOGGER_NAME = "global";
    @Deprecated
    public static final Logger global;
    
    public static final Logger getGlobal() {
        LogManager.getLogManager();
        return Logger.global;
    }
    
    protected Logger(String s, String s2) {
        this(s, s2, null, LogManager.getLogManager(), false);
    }
    
    Logger(String name, String s, Class<?> clazz, LogManager manager, boolean isSystemLogger) {
        this.handlers = new CopyOnWriteArrayList();
        this.loggerBundle = Logger.NO_RESOURCE_BUNDLE;
        this.useParentHandlers = true;
        this.manager = manager;
        this.isSystemLogger = isSystemLogger;
        this.setupResourceInfo(s, clazz);
        this.name = name;
        this.levelValue = Level.INFO.intValue();
    }
    
    private void setCallersClassLoaderRef(Class<?> clazz) {
        ClassLoader referent = (clazz != null) ? clazz.getClassLoader() : null;
        if (referent != null) {
            this.callersClassLoaderRef = new WeakReference(referent);
        }
    }
    
    private ClassLoader getCallersClassLoader() {
        return (this.callersClassLoaderRef != null) ? ((ClassLoader)this.callersClassLoaderRef.get()) : null;
    }
    
    private Logger(String name) {
        this.handlers = new CopyOnWriteArrayList();
        this.loggerBundle = Logger.NO_RESOURCE_BUNDLE;
        this.useParentHandlers = true;
        this.name = name;
        this.isSystemLogger = true;
        this.levelValue = Level.INFO.intValue();
    }
    
    void setLogManager(LogManager manager) {
        this.manager = manager;
    }
    
    private void checkPermission() throws SecurityException {
        if (!this.anonymous) {
            if (this.manager == null) {
                this.manager = LogManager.getLogManager();
            }
            this.manager.checkPermission();
        }
    }
    
    private static Logger demandLogger(String s, String s2, Class<?> clazz) {
        LogManager logManager = LogManager.getLogManager();
        if (System.getSecurityManager() != null && !Logger.SystemLoggerHelper.disableCallerCheck && clazz.getClassLoader() == null) {
            return logManager.demandSystemLogger(s, s2);
        }
        return logManager.demandLogger(s, s2, clazz);
    }
    
    @CallerSensitive
    public static Logger getLogger(String s) {
        return demandLogger(s, null, Reflection.getCallerClass());
    }
    
    @CallerSensitive
    public static Logger getLogger(String s, String s2) {
        Class<Object> callerClass = Reflection.getCallerClass();
        Logger demandLogger = demandLogger(s, s2, callerClass);
        demandLogger.setupResourceInfo(s2, callerClass);
        return demandLogger;
    }
    
    static Logger getPlatformLogger(String s) {
        return LogManager.getLogManager().demandSystemLogger(s, "sun.util.logging.resources.logging");
    }
    
    public static Logger getAnonymousLogger() {
        return getAnonymousLogger(null);
    }
    
    @CallerSensitive
    public static Logger getAnonymousLogger(String s) {
        LogManager logManager = LogManager.getLogManager();
        logManager.drainLoggerRefQueueBounded();
        Logger logger = new Logger(null, s, Reflection.getCallerClass(), logManager, false);
        logger.anonymous = true;
        logger.doSetParent(logManager.getLogger(""));
        return logger;
    }
    
    public ResourceBundle getResourceBundle() {
        return this.findResourceBundle(this.getResourceBundleName(), true);
    }
    
    public String getResourceBundleName() {
        return this.loggerBundle.resourceBundleName;
    }
    
    public void setFilter(Filter filter) throws SecurityException {
        this.checkPermission();
        this.filter = filter;
    }
    
    public Filter getFilter() {
        return this.filter;
    }
    
    public void log(LogRecord logRecord) {
        if (!this.isLoggable(logRecord.getLevel())) {
            return;
        }
        Filter filter = this.filter;
        if (filter != null && !filter.isLoggable(logRecord)) {
            return;
        }
        while (this != null) {
            Handler[] array = (this.isSystemLogger ? this.accessCheckedHandlers() : this.getHandlers());
            for (int length = array.length, i = 0; i < length; ++i) {
                array[i].publish(logRecord);
            }
            if (!(this.isSystemLogger ? this.useParentHandlers : this.getUseParentHandlers())) {
                break;
            }
            this = (this.isSystemLogger ? this.parent : this.getParent());
        }
        return;
    }
    
    private void doLog(LogRecord logRecord) {
        logRecord.setLoggerName(this.name);
        Logger.LoggerBundle effectiveLoggerBundle = this.getEffectiveLoggerBundle();
        ResourceBundle userBundle = effectiveLoggerBundle.userBundle;
        String resourceBundleName = effectiveLoggerBundle.resourceBundleName;
        if (resourceBundleName != null && userBundle != null) {
            logRecord.setResourceBundleName(resourceBundleName);
            logRecord.setResourceBundle(userBundle);
        }
        this.log(logRecord);
    }
    
    public void log(Level level, String s) {
        if (!this.isLoggable(level)) {
            return;
        }
        this.doLog(new LogRecord(level, s));
        return;
    }
    
    public void log(Level level, Supplier<String> supplier) {
        if (!this.isLoggable(level)) {
            return;
        }
        this.doLog(new LogRecord(level, (String)supplier.get()));
        return;
    }
    
    public void log(Level level, String s, Object o) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setParameters(new Object[] { o });
        this.doLog(logRecord);
        return;
    }
    
    public void log(Level level, String s, Object[] parameters) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setParameters(parameters);
        this.doLog(logRecord);
        return;
    }
    
    public void log(Level level, String s, Throwable thrown) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setThrown(thrown);
        this.doLog(logRecord);
        return;
    }
    
    public void log(Level level, Throwable thrown, Supplier<String> supplier) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, (String)supplier.get());
        logRecord.setThrown(thrown);
        this.doLog(logRecord);
        return;
    }
    
    public void logp(Level level, String sourceClassName, String sourceMethodName, String s) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        this.doLog(logRecord);
        return;
    }
    
    public void logp(Level level, String sourceClassName, String sourceMethodName, Supplier<String> supplier) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, (String)supplier.get());
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        this.doLog(logRecord);
        return;
    }
    
    public void logp(Level level, String sourceClassName, String sourceMethodName, String s, Object o) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setParameters(new Object[] { o });
        this.doLog(logRecord);
        return;
    }
    
    public void logp(Level level, String sourceClassName, String sourceMethodName, String s, Object[] parameters) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setParameters(parameters);
        this.doLog(logRecord);
        return;
    }
    
    public void logp(Level level, String sourceClassName, String sourceMethodName, String s, Throwable thrown) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setThrown(thrown);
        this.doLog(logRecord);
        return;
    }
    
    public void logp(Level level, String sourceClassName, String sourceMethodName, Throwable thrown, Supplier<String> supplier) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, (String)supplier.get());
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setThrown(thrown);
        this.doLog(logRecord);
        return;
    }
    
    private void doLog(LogRecord logRecord, String resourceBundleName) {
        logRecord.setLoggerName(this.name);
        if (resourceBundleName != null) {
            logRecord.setResourceBundleName(resourceBundleName);
            logRecord.setResourceBundle(this.findResourceBundle(resourceBundleName, false));
        }
        this.log(logRecord);
    }
    
    private void doLog(LogRecord logRecord, ResourceBundle resourceBundle) {
        logRecord.setLoggerName(this.name);
        if (resourceBundle != null) {
            logRecord.setResourceBundleName(resourceBundle.getBaseBundleName());
            logRecord.setResourceBundle(resourceBundle);
        }
        this.log(logRecord);
    }
    
    @Deprecated
    public void logrb(Level level, String sourceClassName, String sourceMethodName, String s, String s2) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s2);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        this.doLog(logRecord, s);
        return;
    }
    
    @Deprecated
    public void logrb(Level level, String sourceClassName, String sourceMethodName, String s, String s2, Object o) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s2);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setParameters(new Object[] { o });
        this.doLog(logRecord, s);
        return;
    }
    
    @Deprecated
    public void logrb(Level level, String sourceClassName, String sourceMethodName, String s, String s2, Object[] parameters) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s2);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setParameters(parameters);
        this.doLog(logRecord, s);
        return;
    }
    
    public void logrb(Level level, String sourceClassName, String sourceMethodName, ResourceBundle resourceBundle, String s, Object... parameters) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        if (parameters != null && parameters.length != 0) {
            logRecord.setParameters(parameters);
        }
        this.doLog(logRecord, resourceBundle);
        return;
    }
    
    @Deprecated
    public void logrb(Level level, String sourceClassName, String sourceMethodName, String s, String s2, Throwable thrown) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s2);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setThrown(thrown);
        this.doLog(logRecord, s);
        return;
    }
    
    public void logrb(Level level, String sourceClassName, String sourceMethodName, ResourceBundle resourceBundle, String s, Throwable thrown) {
        if (!this.isLoggable(level)) {
            return;
        }
        LogRecord logRecord = new LogRecord(level, s);
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setThrown(thrown);
        this.doLog(logRecord, resourceBundle);
        return;
    }
    
    public void entering(String s, String s2) {
        this.logp(Level.FINER, s, s2, "ENTRY");
    }
    
    public void entering(String s, String s2, Object o) {
        this.logp(Level.FINER, s, s2, "ENTRY {0}", o);
    }
    
    public void entering(String s, String s2, Object[] array) {
        String string = "ENTRY";
        if (array == null) {
            this.logp(Level.FINER, s, s2, string);
            return;
        }
        if (!this.isLoggable(Level.FINER)) {
            return;
        }
        for (int i = 0; i < array.length; ++i) {
            string = string + " {" + i + "}";
        }
        this.logp(Level.FINER, s, s2, string, array);
        return;
    }
    
    public void exiting(String s, String s2) {
        this.logp(Level.FINER, s, s2, "RETURN");
    }
    
    public void exiting(String s, String s2, Object o) {
        this.logp(Level.FINER, s, s2, "RETURN {0}", o);
    }
    
    public void throwing(String sourceClassName, String sourceMethodName, Throwable thrown) {
        if (!this.isLoggable(Level.FINER)) {
            return;
        }
        LogRecord logRecord = new LogRecord(Level.FINER, "THROW");
        logRecord.setSourceClassName(sourceClassName);
        logRecord.setSourceMethodName(sourceMethodName);
        logRecord.setThrown(thrown);
        this.doLog(logRecord);
        return;
    }
    
    public void severe(String s) {
        this.log(Level.SEVERE, s);
    }
    
    public void warning(String s) {
        this.log(Level.WARNING, s);
    }
    
    public void info(String s) {
        this.log(Level.INFO, s);
    }
    
    public void config(String s) {
        this.log(Level.CONFIG, s);
    }
    
    public void fine(String s) {
        this.log(Level.FINE, s);
    }
    
    public void finer(String s) {
        this.log(Level.FINER, s);
    }
    
    public void finest(String s) {
        this.log(Level.FINEST, s);
    }
    
    public void severe(Supplier<String> supplier) {
        this.log(Level.SEVERE, supplier);
    }
    
    public void warning(Supplier<String> supplier) {
        this.log(Level.WARNING, supplier);
    }
    
    public void info(Supplier<String> supplier) {
        this.log(Level.INFO, supplier);
    }
    
    public void config(Supplier<String> supplier) {
        this.log(Level.CONFIG, supplier);
    }
    
    public void fine(Supplier<String> supplier) {
        this.log(Level.FINE, supplier);
    }
    
    public void finer(Supplier<String> supplier) {
        this.log(Level.FINER, supplier);
    }
    
    public void finest(Supplier<String> supplier) {
        this.log(Level.FINEST, supplier);
    }
    
    public void setLevel(Level levelObject) throws SecurityException {
        this.checkPermission();
        synchronized (Logger.treeLock) {
            this.levelObject = levelObject;
            this.updateEffectiveLevel();
        }
    }
    
    final boolean isLevelInitialized() {
        return this.levelObject != null;
    }
    
    public Level getLevel() {
        return this.levelObject;
    }
    
    public boolean isLoggable(Level level) {
        return level.intValue() >= this.levelValue && this.levelValue != Logger.offValue;
    }
    
    public String getName() {
        return this.name;
    }
    
    public void addHandler(Handler e) throws SecurityException {
        e.getClass();
        this.checkPermission();
        this.handlers.add(e);
    }
    
    public void removeHandler(Handler o) throws SecurityException {
        this.checkPermission();
        if (o == null) {
            return;
        }
        this.handlers.remove(o);
        return;
    }
    
    public Handler[] getHandlers() {
        return this.accessCheckedHandlers();
    }
    
    Handler[] accessCheckedHandlers() {
        return (Handler[])this.handlers.toArray(Logger.emptyHandlers);
    }
    
    public void setUseParentHandlers(boolean useParentHandlers) {
        this.checkPermission();
        this.useParentHandlers = useParentHandlers;
    }
    
    public boolean getUseParentHandlers() {
        return this.useParentHandlers;
    }
    
    private static ResourceBundle findSystemResourceBundle(final Locale locale) {
        return (ResourceBundle)AccessController.doPrivileged(new PrivilegedAction<ResourceBundle>() {
            @Override
            public ResourceBundle run() {
                try {
                    return ResourceBundle.getBundle("sun.util.logging.resources.logging", locale);
                }
                catch (MissingResourceException ex) {
                    throw new InternalError(ex.toString());
                }
            }
        });
    }
    
    private synchronized ResourceBundle findResourceBundle(String catalogName, boolean b) {
        if (catalogName == null) {
            return null;
        }
        Locale default1 = Locale.getDefault();
        Logger.LoggerBundle loggerBundle = this.loggerBundle;
        if (loggerBundle.userBundle != null && catalogName.equals(loggerBundle.resourceBundleName)) {
            return loggerBundle.userBundle;
        }
        if (this.catalog != null && default1.equals(this.catalogLocale) && catalogName.equals(this.catalogName)) {
            return this.catalog;
        }
        if (catalogName.equals("sun.util.logging.resources.logging")) {
            this.catalog = findSystemResourceBundle(default1);
            this.catalogName = catalogName;
            this.catalogLocale = default1;
            return this.catalog;
        }
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        if (loader == null) {
            loader = ClassLoader.getSystemClassLoader();
        }
        try {
            this.catalog = ResourceBundle.getBundle(catalogName, default1, loader);
            this.catalogName = catalogName;
            this.catalogLocale = default1;
            return this.catalog;
        }
        catch (MissingResourceException ex) {
            if (b) {
                ClassLoader callersClassLoader = this.getCallersClassLoader();
                if (callersClassLoader == null || callersClassLoader == loader) {
                    return null;
                }
                try {
                    this.catalog = ResourceBundle.getBundle(catalogName, default1, callersClassLoader);
                    this.catalogName = catalogName;
                    this.catalogLocale = default1;
                    return this.catalog;
                }
                catch (MissingResourceException ex2) {
                    return null;
                }
            }
            else {
                return null;
            }
        }
    }
    
    private synchronized void setupResourceInfo(String s, Class<?> callersClassLoaderRef) {
        Logger.LoggerBundle loggerBundle = this.loggerBundle;
        if (loggerBundle.resourceBundleName != null) {
            if (loggerBundle.resourceBundleName.equals(s)) {
                return;
            }
            throw new IllegalArgumentException(loggerBundle.resourceBundleName + " != " + s);
        }
        else {
            if (s == null) {
                return;
            }
            this.setCallersClassLoaderRef(callersClassLoaderRef);
            if (this.isSystemLogger && this.getCallersClassLoader() != null) {
                this.checkPermission();
            }
            if (this.findResourceBundle(s, true) == null) {
                this.callersClassLoaderRef = null;
                throw new MissingResourceException("Can't find " + s + " bundle", s, "");
            }
            assert loggerBundle.userBundle == null;
            this.loggerBundle = Logger.LoggerBundle.get(s, null);
            return;
        }
    }
    
    public void setResourceBundle(ResourceBundle resourceBundle) {
        this.checkPermission();
        String baseBundleName = resourceBundle.getBaseBundleName();
        if (baseBundleName == null || baseBundleName.isEmpty()) {
            throw new IllegalArgumentException("resource bundle must have a name");
        }
        synchronized (this) {
            Logger.LoggerBundle loggerBundle = this.loggerBundle;
            if (loggerBundle.resourceBundleName != null && !loggerBundle.resourceBundleName.equals(baseBundleName)) {
                throw new IllegalArgumentException("can't replace resource bundle");
            }
            this.loggerBundle = Logger.LoggerBundle.get(baseBundleName, resourceBundle);
        }
        return;
    }
    
    public Logger getParent() {
        return this.parent;
    }
    
    public void setParent(Logger logger) {
        if (logger == null) {
            throw new NullPointerException();
        }
        if (this.manager == null) {
            this.manager = LogManager.getLogManager();
        }
        this.manager.checkPermission();
        this.doSetParent(logger);
        return;
    }
    
    private void doSetParent(Logger parent) {
        synchronized (Logger.treeLock) {
            Object e = null;
            if (this.parent != null) {
                Iterator<LogManager.LoggerWeakRef> iterator = this.parent.kids.iterator();
                while (iterator.hasNext()) {
                    e = (Object)iterator.next();
                    if ((Logger)e.get() == this) {
                        iterator.remove();
                        break;
                    }
                    e = null;
                }
            }
            this.parent = parent;
            if (this.parent.kids == null) {
                this.parent.kids = new ArrayList(2);
            }
            if (e == null) {
                LogManager manager = this.manager;
                manager.getClass();
                e = manager.new LoggerWeakRef(this);
            }
            e.setParentRef(new WeakReference(this.parent));
            this.parent.kids.add(e);
            this.updateEffectiveLevel();
        }
    }
    
    final void removeChildLogger(LogManager.LoggerWeakRef loggerWeakRef) {
        synchronized (Logger.treeLock) {
            Iterator<LogManager.LoggerWeakRef> iterator = this.kids.iterator();
            while (iterator.hasNext()) {
                if ((LogManager.LoggerWeakRef)iterator.next() == loggerWeakRef) {
                    iterator.remove();
                    return;
                }
                continue;
            }
        }
    }
    
    private void updateEffectiveLevel() {
        int levelValue;
        if (this.levelObject != null) {
            levelValue = this.levelObject.intValue();
        }
        else if (this.parent != null) {
            levelValue = this.parent.levelValue;
        }
        else {
            levelValue = Level.INFO.intValue();
        }
        if (this.levelValue == levelValue) {
            return;
        }
        this.levelValue = levelValue;
        if (this.kids != null) {
            for (int i = 0; i < this.kids.size(); ++i) {
                Logger logger = (Logger)((LogManager.LoggerWeakRef)this.kids.get(i)).get();
                if (logger != null) {
                    logger.updateEffectiveLevel();
                }
            }
        }
        return;
    }
    
    private Logger.LoggerBundle getEffectiveLoggerBundle() {
        Logger.LoggerBundle loggerBundle = this.loggerBundle;
        if (loggerBundle.isSystemBundle()) {
            return Logger.SYSTEM_BUNDLE;
        }
        ResourceBundle resourceBundle = this.getResourceBundle();
        if (resourceBundle != null && resourceBundle == loggerBundle.userBundle) {
            return loggerBundle;
        }
        if (resourceBundle != null) {
            return Logger.LoggerBundle.get(this.getResourceBundleName(), resourceBundle);
        }
        Logger parent = this.parent;
        while (parent != null) {
            Logger.LoggerBundle loggerBundle2 = parent.loggerBundle;
            if (loggerBundle2.isSystemBundle()) {
                return Logger.SYSTEM_BUNDLE;
            }
            if (loggerBundle2.userBundle != null) {
                return loggerBundle2;
            }
            String s = this.isSystemLogger ? (parent.isSystemLogger ? loggerBundle2.resourceBundleName : null) : parent.getResourceBundleName();
            if (s != null) {
                return Logger.LoggerBundle.get(s, this.findResourceBundle(s, true));
            }
            parent = (this.isSystemLogger ? parent.parent : parent.getParent());
        }
        return Logger.NO_RESOURCE_BUNDLE;
    }
    
    static {
        emptyHandlers = new Handler[0];
        offValue = Level.OFF.intValue();
        SYSTEM_BUNDLE = new Logger.LoggerBundle("sun.util.logging.resources.logging");
        NO_RESOURCE_BUNDLE = new Logger.LoggerBundle();
        treeLock = new Object();
        global = new Logger("global");
    }
    
    private static final class LoggerBundle
    {
        final String resourceBundleName;
        final ResourceBundle userBundle;
        
        private LoggerBundle(String resourceBundleName, ResourceBundle userBundle) {
            this.resourceBundleName = resourceBundleName;
            this.userBundle = userBundle;
        }
        
        boolean isSystemBundle() {
            return "sun.util.logging.resources.logging".equals(this.resourceBundleName);
        }
        
        static Logger.LoggerBundle get(String anObject, ResourceBundle resourceBundle) {
            if (anObject == null && resourceBundle == null) {
                return Logger.NO_RESOURCE_BUNDLE;
            }
            if ("sun.util.logging.resources.logging".equals(anObject) && resourceBundle == null) {
                return Logger.SYSTEM_BUNDLE;
            }
            return new Logger.LoggerBundle(anObject, resourceBundle);
        }
    }
    
    private static class SystemLoggerHelper
    {
        static boolean disableCallerCheck;
        
        private static boolean getBooleanProperty(String s) {
            return (boolean)Boolean.valueOf((String)AccessController.doPrivileged(new PrivilegedAction<String>() {
                @Override
                public String run() {
                    return System.getProperty(s);
                }
            }));
        }
        
        static {
            Logger.SystemLoggerHelper.disableCallerCheck = getBooleanProperty("sun.util.logging.disableCallerCheck");
        }
    }
}
